import os
import subprocess


def find_confg(path):
    pass


class Repo(dict):
    def checkout(self, dest):
        cmd = ['git', 'clone', self['git'], dest]
        proc = subprocess.Popen(cmd)
        proc.wait()


class Config(dict):
    def __init__(self, data):
        self.update(data)

    def repos(self):
        pass

    def ensure_checkout(self, path):
        if len(self['repos']) == 1:
            # single repo project
            repo_data = list(self['repos'].values())[0]
            Repo(repo_data).checkout(path)
        else:
            # mutlie repo project
            repo_path = os.path.join(path, 'src')
            os.makedirs(repo_path)
            for repo in repos.values():
                Rep(repo_data).checkout(repo_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'rueckenwind',
    'docker',
    'pyyaml',
]

test_requirements = [
    'pytest',
    'pytest-tornado',
]

setup(
    name='devenv',
    version='0.1.0',
    description="A development environment based upon docker.",
    long_description=readme + '\n\n' + history,
    author="Florian Ludwig",
    author_email='f.ludwig@greyrook.com',
    url='https://github.com/FlorianLudwig/devenv',
    packages=[
        'devenv',
    ],
    package_dir={'devenv':
                 'devenv'},
    entry_points={
        'console_scripts': [
            'de=devenv.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords='devenv',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import

import os
import urllib.request

import docker
import click
import yaml

from devenv import config

dock = docker.from_env()


def load_config():
    with open('.devenv.yml') as f:
        config = yaml.load(f)

    return config


def get_container_infos(config):
    default_container_config = {
        'network_mode': config.get('network_mode', 'host'),
        'detach': True,
    }

    for container_name in config.get('run', []):
        container_config = {}
        container_config.update(default_container_config)
        container_config.update(config['run'][container_name])
        container_config['name'] = config['project_name'] + '-' + container_name
        if container_config.get('user', None) == '$current':
            container_config['user'] = os.getuid()
        yield container_config


def remove_all(config):
    used_names = [c.name for c in dock.containers.list(all=True)]
    for container_config in get_container_infos(config):
        name = container_config['name']
        if name in used_names:
            click.echo('removing container ' + name)
            dock.containers.get(name).remove(force=True)


def run(config):
    remove_all(config)
    for container_config in get_container_infos(config):
        volumes = {}
        for volume in container_config.get('volumes', []):
            src_path, dst_path = volume.split(':')
            src_path = os.path.abspath(src_path)
            volumes[src_path] = {'bind': dst_path, 'mode': 'rw'}

            click.echo('starting container ' + container_config['name'])
        container_config['volumes'] = volumes

        if container_config.get('user', 'root') not in ('root', 0):
            env = container_config.setdefault('environment', {})
            env.setdefault('HOME', '/tmp/')
        result = dock.containers.run(**container_config)


@click.group()
def main(args=None):
    """Console script for devenv"""


@main.command()
@click.argument('source')
def clone(source):
    if source.startswith('http'):
        fd = urllib.request.urlopen(source)
    elif os.path.exists(source):
        fd = open(source)
    else:
        click.echo('Could not get {}'.format(source))
        return 1

    config_data = yaml.load(fd)
    fd.close()
    cfg = config.Config(config_data)
    cfg.ensure_checkout('.')
    return 0


@main.command()
def dev():
    config = load_config()
    run(config)


@main.command()
def down():
    config = load_config()
    remove_all(config)


if __name__ == "__main__":
    main()
